# coding: utf-8
"""
@file: db_router.py
"""


class DBRouter(object):
    """
    A router to control all database operations on models in the
    auth application.
    """
    db_name = 'other_db'
    route_app_labels = {'other_db'}

    def db_for_read(self, model, **hints):
        """
        Attempts to read auth models go to auth_db.
        """
        if model._meta.app_label in self.route_app_labels:
            return self.db_name
        return None

    def db_for_write(self, model, **hints):
        """
        Attempts to write auth models go to auth_db.
        """
        if model._meta.app_label in self.route_app_labels:
            return self.db_name
        return None

    def allow_relation(self, obj1, obj2, **hints):
        """
        Allow relations if a model in the auth app is involved.
        """
        if (
                obj1._meta.app_label in self.route_app_labels or
                obj2._meta.app_label in self.route_app_labels
        ):
            # if obj1._meta.app_label == 'bigdata' or obj2._meta.app_label == 'bigdata':
            return True
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        """
        Make sure the auth app only appears in the 'auth_db'
        database.
        """
        if app_label in self.route_app_labels:
            return db == self.db_name
        return None
